@@ -219,7 +219,7 @@ var DockedDash = GObject.registerClass({
         // Create a new dash object
         this.dash = new MyDash.MyDash(this._remoteModel, this._monitorIndex);
 
-        if (!settings.get_boolean('show-show-apps-button'))
+        if (Main.overview.isDummy || !settings.get_boolean('show-show-apps-button'))
             this.dash.hideShowAppsButton();
 
         // Create the main actor and the containers for sliding in and out and
@@ -231,15 +231,16 @@ var DockedDash = GObject.registerClass({
             name: 'dashtodockContainer',
             reactive: false,
             style_class: positionStyleClass[this._position],
-            x_align: this._isHorizontal?St.Align.MIDDLE:St.Align.START,
-            y_align: this._isHorizontal?St.Align.START:St.Align.MIDDLE
         });
-        this._delegate = this;
 
         // This is the sliding actor whose allocation is to be tracked for input regions
         this._slider = new DashSlideContainer({
             side: this._position,
-            slidex: 0
+            slidex: 0,
+            x_align: this._isHorizontal ?
+                Clutter.ActorAlign.CENTER : Clutter.ActorAlign.START,
+            y_align: this._isHorizontal ?
+                Clutter.ActorAlign.START : Clutter.ActorAlign.CENTER,
         });
 
         // This is the actor whose hover status us tracked for autohide
@@ -258,45 +259,11 @@ var DockedDash = GObject.registerClass({
         this.dash.add_constraint(this.constrainSize);
 
         this._signalsHandler.add([
-            Main.overview,
-            'item-drag-begin',
-            this._onDragStart.bind(this)
-        ], [
-            Main.overview,
-            'item-drag-end',
-            this._onDragEnd.bind(this)
-        ], [
-            Main.overview,
-            'item-drag-cancelled',
-            this._onDragEnd.bind(this)
-        ], [
             // update when workarea changes, for instance if  other extensions modify the struts
             //(like moving th panel at the bottom)
             global.display,
             'workareas-changed',
             this._resetPosition.bind(this)
-        ], [
-            Main.overview,
-            'showing',
-            this._onOverviewShowing.bind(this)
-        ], [
-            Main.overview,
-            'hiding',
-            this._onOverviewHiding.bind(this)
-        ], [
-            // Hide on appview
-            Main.overview.viewSelector,
-            'page-changed',
-            this._pageChanged.bind(this)
-        ], [
-            Main.overview.viewSelector,
-            'page-empty',
-            this._onPageEmpty.bind(this)
-        ], [
-            // Ensure the ShowAppsButton status is kept in sync
-            Main.overview.viewSelector._showAppsButton,
-            'notify::checked',
-            this._syncShowAppsButtonToggled.bind(this)
         ], [
             global.display,
             'in-fullscreen-changed',
@@ -310,16 +277,7 @@ var DockedDash = GObject.registerClass({
             // Keep dragged icon consistent in size with this dash
             this.dash,
             'icon-size-changed',
-            () => { Main.overview.dashIconSize = this.dash.iconSize; }
-        ], [
-            // This duplicate the similar signal which is in owerview.js.
-            // Being connected and thus executed later this effectively
-            // overwrite any attempt to use the size of the default dash
-            //which given the customization is usually much smaller.
-            // I can't easily disconnect the original signal
-            Main.overview._controls.dash,
-            'icon-size-changed',
-            () => { Main.overview.dashIconSize = this.dash.iconSize; }
+            () => { Main.overview.dash.iconSize = this.dash.iconSize; }
         ], [
             // sync hover after a popupmenu is closed
             this.dash,
@@ -327,6 +285,53 @@ var DockedDash = GObject.registerClass({
             () => { this._box.sync_hover() }
         ]);
 
+        if (!Main.overview.isDummy) {
+            this._signalsHandler.add([
+                Main.overview,
+                'item-drag-begin',
+                this._onDragStart.bind(this)
+            ], [
+                Main.overview,
+                'item-drag-end',
+                this._onDragEnd.bind(this)
+            ], [
+                Main.overview,
+                'item-drag-cancelled',
+                this._onDragEnd.bind(this)
+            ], [
+                Main.overview,
+                'showing',
+                this._onOverviewShowing.bind(this)
+            ], [
+                Main.overview,
+                'hiding',
+                this._onOverviewHiding.bind(this)
+            ], [
+                // Hide on appview
+                Main.overview.viewSelector,
+                'page-changed',
+                this._pageChanged.bind(this)
+            ], [
+                Main.overview.viewSelector,
+                'page-empty',
+                this._onPageEmpty.bind(this)
+            ], [
+                // Ensure the ShowAppsButton status is kept in sync
+                Main.overview.viewSelector._showAppsButton,
+                'notify::checked',
+                this._syncShowAppsButtonToggled.bind(this)
+            ], [
+                // This duplicate the similar signal which is in owerview.js.
+                // Being connected and thus executed later this effectively
+                // overwrite any attempt to use the size of the default dash
+                //which given the customization is usually much smaller.
+                // I can't easily disconnect the original signal
+                Main.overview.dash,
+                'icon-size-changed',
+                () => { Main.overview.dash.iconSize = this.dash.iconSize; }
+            ]);
+        }
+
         this._injectionsHandler = new Utils.InjectionsHandler();
         this._themeManager = new Theming.ThemeManager(this);
 
@@ -356,14 +361,17 @@ var DockedDash = GObject.registerClass({
         this._dashSpacer = new OverviewControls.DashSpacer();
         this._dashSpacer.setDashActor(this._box);
 
-        if (this._position == St.Side.LEFT)
-            Main.overview._controls._group.insert_child_at_index(this._dashSpacer, this._rtl ? -1 : 0); // insert on first
-        else if (this._position ==  St.Side.RIGHT)
-            Main.overview._controls._group.insert_child_at_index(this._dashSpacer, this._rtl ? 0 : -1); // insert on last
-        else if (this._position == St.Side.TOP)
-            Main.overview._overview.insert_child_at_index(this._dashSpacer, 0);
-        else if (this._position == St.Side.BOTTOM)
-            Main.overview._overview.insert_child_at_index(this._dashSpacer, -1);
+        if (!Main.overview.isDummy) {
+            const overviewActor = Main.overview._overview;
+            if (this._position == St.Side.LEFT)
+                overviewActor._controls._group.insert_child_at_index(this._dashSpacer, this._rtl ? -1 : 0); // insert on first
+            else if (this._position ==  St.Side.RIGHT)
+                overviewActor._controls._group.insert_child_at_index(this._dashSpacer, this._rtl ? 0 : -1); // insert on last
+            else if (this._position == St.Side.TOP)
+                overviewActor.insert_child_at_index(this._dashSpacer, 0);
+            else if (this._position == St.Side.BOTTOM)
+                overviewActor.insert_child_at_index(this._dashSpacer, -1);
+        }
 
         // Add dash container actor and the container to the Chrome.
         this.set_child(this._slider);
@@ -400,7 +408,7 @@ var DockedDash = GObject.registerClass({
 
         // Since Gnome 3.8 dragging an app without having opened the overview before cause the attemp to
         //animate a null target since some variables are not initialized when the viewSelector is created
-        if (Main.overview.viewSelector._activePage == null)
+        if (!Main.overview.isDummy && Main.overview.viewSelector._activePage == null)
             Main.overview.viewSelector._activePage = Main.overview.viewSelector._workspacesPage;
 
         this._updateVisibilityMode();
@@ -487,7 +495,8 @@ var DockedDash = GObject.registerClass({
             settings,
             'changed::show-show-apps-button',
             () => {
-                    if (settings.get_boolean('show-show-apps-button'))
+                    if (!Main.overview.isDummy &&
+                        settings.get_boolean('show-show-apps-button'))
                         this.dash.showShowAppsButton();
                     else
                         this.dash.hideShowAppsButton();
@@ -681,15 +690,14 @@ var DockedDash = GObject.registerClass({
         // If no hiding animation is running or queued
         if ((this._dockState == State.SHOWN) || (this._dockState == State.SHOWING)) {
             let settings = DockManager.settings;
-            let delay;
+            let delay = settings.get_double('hide-delay');
 
-            if (this._dockState == State.SHOWING)
-                //if a show already started, let it finish; queue hide without removing the show.
-                // to obtain this I increase the delay to avoid the overlap and interference
-                // between the animations
-                delay = settings.get_double('hide-delay') + settings.get_double('animation-time');
-            else
-                delay = settings.get_double('hide-delay');
+            if (this._dockState == State.SHOWING) {
+                // if a show already started, let it finish; queue hide without removing the show.
+                // to obtain this, we wait for the animateIn animation to be completed
+                this._delayedHide = true;
+                return;
+            }
 
             this.emit('hiding');
             this._animateOut(settings.get_double('animation-time'), delay);
@@ -698,6 +706,7 @@ var DockedDash = GObject.registerClass({
 
     _animateIn(time, delay) {
         this._dockState = State.SHOWING;
+        delete this._delayedHide;
 
         this._slider.ease_property('slidex', 1, {
             duration: time * 1000,
@@ -710,8 +719,13 @@ var DockedDash = GObject.registerClass({
                 // gives users an opportunity to hover over the dock
                 if (this._removeBarrierTimeoutId > 0)
                     GLib.source_remove(this._removeBarrierTimeoutId);
-                this._removeBarrierTimeoutId = GLib.timeout_add(
-                    GLib.PRIORITY_DEFAULT, 100, this._removeBarrier.bind(this));
+
+                if (!this._delayedHide) {
+                    this._removeBarrierTimeoutId = GLib.timeout_add(
+                        GLib.PRIORITY_DEFAULT, 100, this._removeBarrier.bind(this));
+                } else {
+                    this._hide();
+                }
             }
         });
     }
@@ -979,16 +993,17 @@ var DockedDash = GObject.registerClass({
                 direction = Meta.BarrierDirection.NEGATIVE_Y;
             }
 
-            this._barrier = new Meta.Barrier({
-                display: global.display,
-                x1: x1,
-                x2: x2,
-                y1: y1,
-                y2: y2,
-                directions: direction
-            });
-            if (this._pressureBarrier)
+            if (this._pressureBarrier && this._dockState == State.HIDDEN) {
+                this._barrier = new Meta.Barrier({
+                    display: global.display,
+                    x1: x1,
+                    x2: x2,
+                    y1: y1,
+                    y2: y2,
+                    directions: direction
+                });
                 this._pressureBarrier.addBarrier(this._barrier);
+            }
         }
     }
 
@@ -1076,8 +1091,6 @@ var DockedDash = GObject.registerClass({
                 this.remove_style_class_name('extended');
             }
         }
-
-        this._y0 = this.y;
     }
 
     // Set the dash at the correct depth in z
@@ -1141,7 +1154,7 @@ var DockedDash = GObject.registerClass({
          * Moreover, hiding the spacer ensure the appGrid allocaton is triggered.
          * This matter as the appview spring animation is triggered by to first reallocaton of the appGrid,
          * (See appDisplay.js, line 202 on GNOME Shell 3.14:
-         *                             this._grid.actor.connect('notify::allocation', ...)
+         *                             this._grid.connect('notify::allocation', ...)
          * which in turn seems to be triggered by changes in the other actors in the overview.
          * Normally, as far as I could understand, either the dashSpacer being hidden or the workspacesThumbnails
          * sliding out would trigger the allocation. However, with no stock dash
@@ -1261,19 +1274,30 @@ var DockedDash = GObject.registerClass({
                 ws = activeWs.get_neighbor(direction)
 
                 if (Main.wm._workspaceSwitcherPopup == null)
-                    Main.wm._workspaceSwitcherPopup = new WorkspaceSwitcherPopup.WorkspaceSwitcherPopup();
+                    // Support Workspace Grid extension showing their custom Grid Workspace Switcher
+                    if (Utils.DisplayWrapper.getWorkspaceManager().workspace_grid !== undefined)
+                        Main.wm._workspaceSwitcherPopup = Utils.DisplayWrapper.getWorkspaceManager()
+                            .workspace_grid.getWorkspaceSwitcherPopup();
+                    else
+                        Main.wm._workspaceSwitcherPopup = new WorkspaceSwitcherPopup.WorkspaceSwitcherPopup();
                     // Set the actor non reactive, so that it doesn't prevent the
                     // clicks events from reaching the dash actor. I can't see a reason
                     // why it should be reactive.
-                    Main.wm._workspaceSwitcherPopup.actor.reactive = false;
+                    Main.wm._workspaceSwitcherPopup.reactive = false;
                     Main.wm._workspaceSwitcherPopup.connect('destroy', function() {
                         Main.wm._workspaceSwitcherPopup = null;
                     });
 
+                // If Workspace Grid is installed, let them handle the scroll behaviour.
+                if (Utils.DisplayWrapper.getWorkspaceManager().workspace_grid !== undefined)
+                    ws = Utils.DisplayWrapper.getWorkspaceManager().workspace_grid
+                        .actionMoveWorkspace(direction);
+                else
+                    Main.wm.actionMoveWorkspace(ws);
+
                 // Do not show wokspaceSwithcer in overview
                 if (!Main.overview.visible)
                     Main.wm._workspaceSwitcherPopup.display(direction, ws.index());
-                Main.wm.actionMoveWorkspace(ws);
 
                 return true;
             }
@@ -1285,13 +1309,12 @@ var DockedDash = GObject.registerClass({
     _activateApp(appIndex) {
         let children = this.dash._box.get_children().filter(function(actor) {
                 return actor.child &&
-                       actor.child._delegate &&
-                       actor.child._delegate.app;
+                       actor.child.app;
         });
 
         // Apps currently in the dash
         let apps = children.map(function(actor) {
-                return actor.child._delegate;
+                return actor.child;
             });
 
         // Activate with button = 1, i.e. same as left click
@@ -1425,7 +1448,7 @@ var KeyboardShortcuts = class DashToDock_KeyboardShortcuts {
     _showOverlay() {
         for (let i = 0; i < this._allDocks.length; i++) {
             let dock = this._allDocks[i];
-            if (dock._settings.get_boolean('hotkeys-overlay'))
+            if (DockManager.settings.get_boolean('hotkeys-overlay'))
                 dock.dash.toggleNumberOverlay(true);
 
             // Restart the counting if the shortcut is pressed again
@@ -1435,7 +1458,7 @@ var KeyboardShortcuts = class DashToDock_KeyboardShortcuts {
             }
 
             // Hide the overlay/dock after the timeout
-            let timeout = dock._settings.get_double('shortcut-timeout') * 1000;
+            let timeout = DockManager.settings.get_double('shortcut-timeout') * 1000;
             dock._numberOverlayTimeoutId = GLib.timeout_add(
                 GLib.PRIORITY_DEFAULT, timeout, () => {
                     dock._numberOverlayTimeoutId = 0;
@@ -1445,7 +1468,7 @@ var KeyboardShortcuts = class DashToDock_KeyboardShortcuts {
             });
 
             // Show the dock if it is hidden
-            if (dock._settings.get_boolean('hotkeys-show-dock')) {
+            if (DockManager.settings.get_boolean('hotkeys-show-dock')) {
                 let showDock = (dock._intellihideIsEnabled || dock._autohideIsEnabled);
                 if (showDock)
                     dock._show();
@@ -1724,10 +1747,13 @@ var DockManager = class DashToDock_DockManager {
         Main.overview._dash = this._allDocks[0].dash;
 
         // set stored icon size  to the new dash
-        Main.overview.dashIconSize = this._allDocks[0].dash.iconSize;
+        Main.overview.dash.iconSize = this._allDocks[0].dash.iconSize;
+
+        if (Main.overview.isDummy)
+            return;
 
         // Hide usual Dash
-        Main.overview._controls.dash.actor.hide();
+        Main.overview.dash.hide();
 
         // Also set dash width to 1, so it's almost not taken into account by code
         // calculaing the reserved space in the overview. The reason to keep it at 1 is
@@ -1735,7 +1761,7 @@ var DockManager = class DashToDock_DockManager {
         // in turn is triggergin the appsIcon spring animation, required when no other
         // actors has this effect, i.e in horizontal mode and without the workspaceThumnails
         // 1 static workspace only)
-        Main.overview._controls.dash.actor.set_width(1);
+        Main.overview.dash.set_width(1);
     }
 
     _deleteDocks() {
@@ -1752,13 +1778,16 @@ var DockManager = class DashToDock_DockManager {
     }
 
     _restoreDash() {
-        Main.overview._controls.dash.actor.show();
-        Main.overview._controls.dash.actor.set_width(-1); //reset default dash size
+        if (Main.overview.isDummy)
+            return;
+
+        Main.overview.dash.show();
+        Main.overview.dash.set_width(-1); //reset default dash size
         // This force the recalculation of the icon size
-        Main.overview._controls.dash._maxHeight = -1;
+        Main.overview.dash._maxHeight = -1;
 
         // reset stored icon size  to the default dash
-        Main.overview.dashIconSize = Main.overview._controls.dash.iconSize;
+        Main.overview.dash.iconSize = Main.overview.dash.iconSize;
 
         Main.overview._dash = this._oldDash;
     }
@@ -1777,7 +1806,7 @@ var DockManager = class DashToDock_DockManager {
             // find visible view
             let visibleView;
             Main.overview.viewSelector.appDisplay._views.every(function(v, index) {
-                if (v.view.actor.visible) {
+                if (v.view.visible) {
                     visibleView = index;
                     return false;
                 }
@@ -1890,16 +1919,16 @@ var DockManager = class DashToDock_DockManager {
                              this._preferredMonitorIndex == Main.layoutManager.primaryIndex;
 
         if (!isHorizontal && dockOnPrimary && extendHeight && fixedIsEnabled) {
-            Main.panel._rightCorner.actor.hide();
-            Main.panel._leftCorner.actor.hide();
+            Main.panel._rightCorner.hide();
+            Main.panel._leftCorner.hide();
         }
         else
             this._revertPanelCorners();
     }
 
     _revertPanelCorners() {
-        Main.panel._leftCorner.actor.show();
-        Main.panel._rightCorner.actor.show();
+        Main.panel._leftCorner.show();
+        Main.panel._rightCorner.show();
     }
 };
 Signals.addSignalMethods(DockManager.prototype);
